@@ -10,33 +10,33 @@ die () {
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-2.6.9-67.0.7.EL.src.rpm ]
+if [ ! -r kernel-2.6.9-67.0.22.EL.src.rpm ]
 then
-    wget http://ftp.riken.jp/Linux/centos/4.6/updates/SRPMS/kernel-2.6.9-67.0.7.EL.src.rpm || die "Can't download source package."
+    wget http://ftp.riken.jp/Linux/centos/4.6/updates/SRPMS/kernel-2.6.9-67.0.22.EL.src.rpm || die "Can't download source package."
 fi
-rpm -ivh kernel-2.6.9-67.0.7.EL.src.rpm || die "Can't install source package."
+rpm -ivh kernel-2.6.9-67.0.22.EL.src.rpm || die "Can't install source package."
 
 cd /usr/src/redhat/SOURCES/ || die "Can't chdir to /usr/src/redhat/SOURCES/ ."
-if [ ! -r ccs-patch-1.5.4-20080510.tar.gz ]
+if [ ! -r ccs-patch-1.5.5-20080903.tar.gz ]
 then
-    wget http://osdn.dl.sourceforge.jp/tomoyo/27219/ccs-patch-1.5.4-20080510.tar.gz || die "Can't download patch."
+    wget http://osdn.dl.sourceforge.jp/tomoyo/27219/ccs-patch-1.5.5-20080903.tar.gz || die "Can't download patch."
 fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 cp -p /usr/src/redhat/SPECS/kernel-2.6.spec . || die "Can't copy spec file."
 patch << "EOF" || die "Can't patch spec file."
---- kernel-2.6.spec	2008-03-15 18:48:40.000000000 +0900
-+++ kernel-2.6.spec	2008-04-16 13:19:28.000000000 +0900
-@@ -30,7 +30,7 @@
+--- kernel-2.6.spec	2008-07-24 05:57:53.000000000 +0900
++++ kernel-2.6.spec	2008-07-25 09:11:02.000000000 +0900
+@@ -26,7 +26,7 @@
  # that the kernel isn't the stock distribution kernel, for example by
  # adding some text to the end of the version number.
  #
--%define release 67.0.7.EL
-+%define release 67.0.7.EL_tomoyo_1.5.4
+-%define release 67.0.22.EL
++%define release 67.0.22.EL_tomoyo_1.5.5
  %define sublevel 9
  %define kversion 2.6.%{sublevel}
  %define rpmversion 2.6.%{sublevel}
-@@ -136,6 +136,9 @@
+@@ -132,6 +132,9 @@
  # to versions below the minimum
  #
  
@@ -46,7 +46,7 @@ patch << "EOF" || die "Can't patch spec file."
  #
  # First the general kernel 2.6 required versions as per
  # Documentation/Changes
-@@ -172,7 +175,7 @@
+@@ -168,7 +171,7 @@
  %define __find_provides /usr/lib/rpm/redhat/find-kmod-provides.sh
  %define __find_requires %{nil}
  
@@ -55,19 +55,19 @@ patch << "EOF" || die "Can't patch spec file."
  Group: System Environment/Kernel
  License: GPLv2
  Version: %{rpmversion}
-@@ -3749,6 +3752,11 @@
+@@ -3796,6 +3799,11 @@
  
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.5.4-20080510.tar.gz
-+sed -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -67.0.7.EL/" -- Makefile
-+patch -sp1 < patches/ccs-patch-2.6.9-67.0.7.EL.diff
++tar -zxf %_sourcedir/ccs-patch-1.5.5-20080903.tar.gz
++# sed -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -67.0.22.EL/" -- Makefile
++patch -sp1 < patches/ccs-patch-2.6.9-centos-4.6.diff
 +
  cp %{SOURCE10} Documentation/
  
  mkdir configs
-@@ -3760,6 +3768,9 @@
+@@ -3807,6 +3815,9 @@
  for i in *.config 
  do 
  	mv $i .config 
@@ -78,11 +78,11 @@ patch << "EOF" || die "Can't patch spec file."
  	cp .config configs/$i 
  done
 EOF
-mv kernel-2.6.spec kernel-2.6.9-67.0.7.EL_tomoyo_1.5.4.spec || die "Can't rename spec file."
+mv kernel-2.6.spec ccs-kernel.spec || die "Can't rename spec file."
 echo ""
 echo ""
 echo ""
-echo "Edit /tmp/kernel-2.6.9-67.0.7.EL_tomoyo_1.5.4.spec if needed, and run"
-echo "rpmbuild -bb /tmp/kernel-2.6.9-67.0.7.EL_tomoyo_1.5.4.spec"
+echo "Edit /tmp/ccs-kernel.spec if needed, and run"
+echo "rpmbuild -bb /tmp/ccs-kernel.spec"
 echo "to build kernel rpm packages."
 exit 0
