@@ -1,12 +1,11 @@
-
 /*
  * fs/proc/ccs_proc.c
  *
  * /proc interface for SAKURA and TOMOYO.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.4   2007/04/01
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -24,7 +23,7 @@
 
 #if defined(CONFIG_SAKURA) || defined(CONFIG_TOMOYO)
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,23)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 23)
 static inline struct proc_dir_entry *PDE(const struct inode *inode)
 {
 	return (struct proc_dir_entry *) inode->u.generic_ip;
@@ -73,40 +72,39 @@ static __init void CreateEntry(const char *name, const mode_t mode, struct proc_
 	}
 }
 
-void __init CCSProc_Init(void)
+static int __init ccs_proc_init(void)
 {
 	struct proc_dir_entry *ccs_dir = proc_mkdir("ccs", NULL);
-	struct proc_dir_entry *policy_dir = proc_mkdir("policy", ccs_dir),
-		*info_dir = proc_mkdir("info", ccs_dir);
-	extern void __init realpath_Init(void);
-	realpath_Init();
-	FindDomain(""); /* Set domainname of KERNEL domain. */
-	CreateEntry("query",            0600, policy_dir, CCS_POLICY_QUERY);
+	CreateEntry("query",            0600, ccs_dir, CCS_QUERY);
 #ifdef CONFIG_SAKURA
-	CreateEntry("system_policy",    0600, policy_dir, CCS_POLICY_SYSTEMPOLICY);
+	CreateEntry("system_policy",    0600, ccs_dir, CCS_SYSTEMPOLICY);
 #endif
 #ifdef CONFIG_TOMOYO
-	CreateEntry("domain_policy",    0600, policy_dir, CCS_POLICY_DOMAINPOLICY);
-	CreateEntry("exception_policy", 0600, policy_dir, CCS_POLICY_EXCEPTIONPOLICY);
-	CreateEntry(".domain_status",   0600, policy_dir, CCS_POLICY_DOMAIN_STATUS);
-	CreateEntry(".process_status",  0400, info_dir, CCS_INFO_PROCESS_STATUS);
-#ifdef CONFIG_TOMOYO_AUDIT
-	CreateEntry("grant_log",        0400, info_dir, CCS_INFO_GRANTLOG);
-	CreateEntry("reject_log",       0400, info_dir, CCS_INFO_REJECTLOG);
+	CreateEntry("domain_policy",    0600, ccs_dir, CCS_DOMAINPOLICY);
+	CreateEntry("exception_policy", 0600, ccs_dir, CCS_EXCEPTIONPOLICY);
+	CreateEntry("grant_log",        0400, ccs_dir, CCS_GRANTLOG);
+	CreateEntry("reject_log",       0400, ccs_dir, CCS_REJECTLOG);
 #endif
-	CreateEntry("self_domain",      0400, info_dir, CCS_INFO_SELFDOMAIN);
-#ifdef CONFIG_TOMOYO_MAC_FOR_FILE
-	CreateEntry("mapping",          0400, info_dir, CCS_INFO_MAPPING);
-#endif
-#endif
-	CreateEntry("meminfo",          0400, info_dir, CCS_INFO_MEMINFO);
-	CreateEntry("status",           0600, ccs_dir, CCS_STATUS);
-	CreateEntry("manager",          0600, policy_dir, CCS_POLICY_MANAGER);
-	CreateEntry(".updates_counter", 0400, info_dir, CCS_INFO_UPDATESCOUNTER);
+	CreateEntry("self_domain",      0400, ccs_dir, CCS_SELFDOMAIN);
+	CreateEntry(".domain_status",   0600, ccs_dir, CCS_DOMAIN_STATUS);
+	CreateEntry(".process_status",  0400, ccs_dir, CCS_PROCESS_STATUS);
+	CreateEntry("meminfo",          0400, ccs_dir, CCS_MEMINFO);
+	CreateEntry("profile",          0600, ccs_dir, CCS_PROFILE);
+	CreateEntry("manager",          0600, ccs_dir, CCS_MANAGER);
+	CreateEntry(".updates_counter", 0400, ccs_dir, CCS_UPDATESCOUNTER);
+	CreateEntry("version",          0400, ccs_dir, CCS_VERSION);
+	return 0;
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+__initcall(ccs_proc_init);
 #else
-void __init CCSProc_Init(void) {}
+core_initcall(ccs_proc_init);
 #endif
 
-EXPORT_SYMBOL(CCSProc_Init);
+#endif
+
+/* For compatibility with 1.4.x/1.5.x patches. */
+void __init CCSProc_Init(void)
+{
+}
