@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.5.4   2008/05/10
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -26,9 +26,15 @@
 #include <linux/namei.h>
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+#define PATH_or_NAMEIDATA path
+#else
+#define PATH_or_NAMEIDATA nameidata
+#endif
+
 /*************************  CONCEAL MOUNT PROTECTOR  *************************/
 
-int SAKURA_MayMount(struct nameidata *nd)
+int SAKURA_MayMount(struct PATH_or_NAMEIDATA *path)
 {
 	int flag = 0;
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
@@ -44,51 +50,42 @@ int SAKURA_MayMount(struct nameidata *nd)
 		list_for_each(p, &namespace->list) {
 			struct vfsmount *vfsmnt = list_entry(p, struct vfsmount, mnt_list);
 			struct dentry *dentry = vfsmnt->mnt_root;
-			spin_lock(&dcache_lock);
+			ccs_realpath_lock();
 			if (IS_ROOT(dentry) || !d_unhashed(dentry)) {
 				while (1) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-					if (nd->path.mnt->mnt_root == vfsmnt->mnt_root && nd->path.dentry == dentry) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 26)
+					if (path->path.mnt->mnt_root == vfsmnt->mnt_root && path->path.dentry == dentry) {
 						flag = 1;
 						break;
 					}
 #else
-					if (nd->mnt->mnt_root == vfsmnt->mnt_root && nd->dentry == dentry) {
+					if (path->mnt->mnt_root == vfsmnt->mnt_root && path->dentry == dentry) {
 						flag = 1;
 						break;
 					}
 #endif
 					if (dentry == vfsmnt->mnt_root || IS_ROOT(dentry)) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-						spin_lock(&vfsmount_lock);
-#endif
 						if (vfsmnt->mnt_parent == vfsmnt) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-							spin_unlock(&vfsmount_lock);
-#endif
 							break;
 						}
 						dentry = vfsmnt->mnt_mountpoint;
 						vfsmnt = vfsmnt->mnt_parent;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-						spin_unlock(&vfsmount_lock);
-#endif
 						continue;
 					}
 					dentry = dentry->d_parent;
 				}
 			}
-			spin_unlock(&dcache_lock);
+			ccs_realpath_unlock();
 			if (flag) break;
 		}
 	}
 	if (flag) {
 		int error = -EPERM;
 		const int is_enforce = CheckCCSEnforce(CCS_SAKURA_DENY_CONCEAL_MOUNT);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-		const char *dir = realpath_from_dentry(nd->path.dentry, nd->path.mnt);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 26)
+		const char *dir = realpath_from_dentry(path->path.dentry, path->path.mnt);
 #else
-		const char *dir = realpath_from_dentry(nd->dentry, nd->mnt);
+		const char *dir = realpath_from_dentry(path->dentry, path->mnt);
 #endif
 		if (dir) {
 			const char *exename = GetEXE();
