@@ -3,9 +3,9 @@
  *
  * Implementation of the Tamper-Proof Device Filesystem.
  *
- * Portions Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Portions Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.4   2007/04/01
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -50,6 +50,8 @@
 
 #include <linux/namei.h>
 #include <linux/version.h>
+#include <linux/sched.h>
+#include <linux/mm.h>
 
 static struct super_operations syaoran_ops;
 static struct address_space_operations syaoran_aops;
@@ -60,7 +62,7 @@ static struct file_operations syaoran_file_operations;
 
 static struct backing_dev_info syaoran_backing_dev_info = {
 	.ra_pages = 0,        /* No readahead */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,12)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
 	.memory_backed  = 1,    /* Does not contribute to dirty memory */
 #else
 	.capabilities = BDI_CAP_NO_ACCT_DIRTY | BDI_CAP_NO_WRITEBACK |
@@ -71,7 +73,7 @@ static struct backing_dev_info syaoran_backing_dev_info = {
 
 #include <linux/syaoran.h>
 
-struct inode *syaoran_get_inode(struct super_block *sb, int mode, dev_t dev)
+static struct inode *syaoran_get_inode(struct super_block *sb, int mode, dev_t dev)
 {
 	struct inode * inode = new_inode(sb);
 
@@ -79,8 +81,10 @@ struct inode *syaoran_get_inode(struct super_block *sb, int mode, dev_t dev)
 		inode->i_mode = mode;
 		inode->i_uid = current->fsuid;
 		inode->i_gid = current->fsgid;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
+#if !defined(RHEL_MAJOR) || RHEL_MAJOR != 5
 		inode->i_blksize = PAGE_CACHE_SIZE;
+#endif
 #endif
 		inode->i_blocks = 0;
 		inode->i_mapping->a_ops = &syaoran_aops;
@@ -116,8 +120,7 @@ struct inode *syaoran_get_inode(struct super_block *sb, int mode, dev_t dev)
  * File creation. Allocate an inode, and we're done..
  */
 /* SMP-safe */
-static int
-syaoran_mknod(struct inode *dir, struct dentry *dentry, int mode, dev_t dev)
+static int syaoran_mknod(struct inode *dir, struct dentry *dentry, int mode, dev_t dev)
 {
 	struct inode * inode;
 	int error = -ENOSPC;
@@ -210,14 +213,29 @@ static int syaoran_setattr(struct dentry * dentry, struct iattr * attr)
 	return error;
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
 static struct address_space_operations syaoran_aops = {
 	.readpage       = simple_readpage,
 	.prepare_write  = simple_prepare_write,
 	.commit_write   = simple_commit_write
 };
+#else
+static int syaoran_set_page_dirty_no_writeback(struct page *page)
+{
+	if (!PageDirty(page))
+		SetPageDirty(page);
+	return 0;
+}
+static struct address_space_operations syaoran_aops = {
+	.readpage       = simple_readpage,
+	.write_begin    = simple_write_begin,
+	.write_end      = simple_write_end,
+	.set_page_dirty = syaoran_set_page_dirty_no_writeback,
+};
+#endif
 
 static struct file_operations syaoran_file_operations = {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 	.read       = generic_file_read,
 	.write      = generic_file_write,
 #else
@@ -228,7 +246,11 @@ static struct file_operations syaoran_file_operations = {
 #endif
 	.mmap       = generic_file_mmap,
 	.fsync      = simple_sync_file,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 	.sendfile   = generic_file_sendfile,
+#else
+	.splice_read    = generic_file_splice_read,
+#endif
 	.llseek     = generic_file_llseek,
 };
 
@@ -273,7 +295,7 @@ static int syaoran_fill_super(struct super_block * sb, void * data, int silent)
 	sb->s_blocksize_bits = PAGE_CACHE_SHIFT;
 	sb->s_magic = SYAORAN_MAGIC;
 	sb->s_op = &syaoran_ops;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,9)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 9)
 	sb->s_time_gran = 1;
 #endif
 	{
@@ -294,14 +316,14 @@ static int syaoran_fill_super(struct super_block * sb, void * data, int silent)
 	return 0;
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
-struct super_block *syaoran_get_sb(struct file_system_type *fs_type,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
+static struct super_block *syaoran_get_sb(struct file_system_type *fs_type,
 	int flags, const char *dev_name, void *data)
 {
 	return get_sb_nodev(fs_type, flags, data, syaoran_fill_super);
 }
 #else
-int syaoran_get_sb(struct file_system_type *fs_type,
+static int syaoran_get_sb(struct file_system_type *fs_type,
 	int flags, const char *dev_name, void *data, struct vfsmount *mnt)
 {
 	return get_sb_nodev(fs_type, flags, data, syaoran_fill_super, mnt);
@@ -325,9 +347,7 @@ static void __exit exit_syaoran_fs(void)
 	unregister_filesystem(&syaoran_fs_type);
 }
 
-module_init(init_syaoran_fs)
-module_exit(exit_syaoran_fs)
+module_init(init_syaoran_fs);
+module_exit(exit_syaoran_fs);
 
 MODULE_LICENSE("GPL");
-
-EXPORT_SYMBOL(syaoran_get_sb);
