@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Based Mandatory Access Control.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.4   2007/04/01
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -24,49 +24,48 @@ extern struct semaphore domain_acl_lock;
 
 /***** The structure for globally readable files. *****/
 
-typedef struct globally_readable_file_entry {
+struct globally_readable_file_entry {
 	struct globally_readable_file_entry *next;
 	const struct path_info *filename;
 	int is_deleted;
-} GLOBALLY_READABLE_FILE_ENTRY;
+};
 
 /***** The structure for filename patterns. *****/
 
-typedef struct pattern_entry {
+struct pattern_entry {
 	struct pattern_entry *next;
 	const struct path_info *pattern;
 	int is_deleted;
-} PATTERN_ENTRY;
+};
 
 /***** The structure for non-rewritable-by-default file patterns. *****/
 
-typedef struct no_rewrite_entry {
+struct no_rewrite_entry {
 	struct no_rewrite_entry *next;
 	const struct path_info *pattern;
 	int is_deleted;
-} NO_REWRITE_ENTRY;
+};
 
 /***** The structure for detailed write operations. *****/
 
 static struct {
 	const char *keyword;
 	const int paths;
-	int check_type;
 } acl_type_array[] = { /* mapping.txt */
-	{ "create",   1, 1 }, // TYPE_CREATE_ACL
-	{ "unlink",   1, 1 }, // TYPE_UNLINK_ACL
-	{ "mkdir",    1, 1 }, // TYPE_MKDIR_ACL
-	{ "rmdir",    1, 1 }, // TYPE_RMDIR_ACL
-	{ "mkfifo",   1, 1 }, // TYPE_MKFIFO_ACL
-	{ "mksock",   1, 1 }, // TYPE_MKSOCK_ACL
-	{ "mkblock",  1, 1 }, // TYPE_MKBLOCK_ACL
-	{ "mkchar",   1, 1 }, // TYPE_MKCHAR_ACL
-	{ "truncate", 1, 1 }, // TYPE_TRUNCATE_ACL
-	{ "symlink",  1, 1 }, // TYPE_SYMLINK_ACL
-	{ "link",     2, 1 }, // TYPE_LINK_ACL
-	{ "rename",   2, 1 }, // TYPE_RENAME_ACL
-	{ "rewrite",  1, 1 }, // TYPE_REWRITE_ACL
-	{ NULL, 0, 0 }
+	{ "create",   1 }, /* TYPE_CREATE_ACL   */
+	{ "unlink",   1 }, /* TYPE_UNLINK_ACL   */
+	{ "mkdir",    1 }, /* TYPE_MKDIR_ACL    */
+	{ "rmdir",    1 }, /* TYPE_RMDIR_ACL    */
+	{ "mkfifo",   1 }, /* TYPE_MKFIFO_ACL   */
+	{ "mksock",   1 }, /* TYPE_MKSOCK_ACL   */
+	{ "mkblock",  1 }, /* TYPE_MKBLOCK_ACL  */
+	{ "mkchar",   1 }, /* TYPE_MKCHAR_ACL   */
+	{ "truncate", 1 }, /* TYPE_TRUNCATE_ACL */
+	{ "symlink",  1 }, /* TYPE_SYMLINK_ACL  */
+	{ "link",     2 }, /* TYPE_LINK_ACL     */
+	{ "rename",   2 }, /* TYPE_RENAME_ACL   */
+	{ "rewrite",  1 }, /* TYPE_REWRITE_ACL  */
+	{ NULL, 0 }
 };
 
 /*************************  UTILITY FUNCTIONS  *************************/
@@ -83,14 +82,6 @@ int acltype2paths(const unsigned int acl_type)
 		? acl_type_array[acl_type].paths : 0;
 }
 
-static unsigned int CheckACLFlags(const unsigned int index)
-{
-	if (index < (sizeof(acl_type_array) / sizeof(acl_type_array[0])) - 1)
-		return acl_type_array[index].check_type;
-	printk("%s: Index %u is out of range. Fix the kernel source.\n", __FUNCTION__, index);
-	return 0;
-}
-
 static int strendswith(const char *name, const char *tail)
 {
 	int len;
@@ -101,12 +92,12 @@ static int strendswith(const char *name, const char *tail)
 
 static struct path_info *GetPath(struct dentry *dentry, struct vfsmount *mnt)
 {
-	struct path_info_with_data { /* sizeof(struct path_info_with_data) <= PAGE_SIZE */
+	struct path_info_with_data { /* Keep sizeof(struct path_info_with_data) <= PAGE_SIZE for speed. */
 		struct path_info head; /* Keep this first, for this pointer is passed to ccs_free(). */
 		char bariier1[16];
 		char body[CCS_MAX_PATHNAME_LEN];
 		char barrier2[16];
-	} *buf = (struct path_info_with_data *) ccs_alloc(sizeof(struct path_info_with_data));
+	} *buf = ccs_alloc(sizeof(*buf));
 	if (buf) {
 		int error;
 		if ((error = realpath_from_dentry2(dentry, mnt, buf->body, sizeof(buf->body) - 1)) == 0) {
@@ -115,19 +106,17 @@ static struct path_info *GetPath(struct dentry *dentry, struct vfsmount *mnt)
 			return &buf->head;
 		}
 		ccs_free(buf); buf = NULL;
-		printk("realpath_from_dentry = %d\n", error);
 	}
 	return NULL;
 }
 
 /*************************  PROTOTYPES  *************************/
 
-static int AddDoubleWriteACL(const u8 type, const char *filename1, const char *filename2, struct domain_info * const domain, const u8 is_add, const struct condition_list *condition);
-static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_info * const domain, const u8 is_add, const struct condition_list *condition);
+static int AddDoubleWriteACL(const u8 type, const char *filename1, const char *filename2, struct domain_info * const domain, const struct condition_list *condition, const u8 is_delete);
+static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_info * const domain, const struct condition_list *condition, const u8 is_delete);
 
 /*************************  AUDIT FUNCTIONS  *************************/
 
-#ifdef CONFIG_TOMOYO_AUDIT
 static int AuditFileLog(const struct path_info *filename, const u8 perm, const int is_granted)
 {
 	char *buf;
@@ -149,59 +138,14 @@ static int AuditWriteLog(const char *operation, const struct path_info *filename
 	snprintf(buf + strlen(buf), len - strlen(buf) - 1, "allow_%s %s %s\n", operation, filename1->name, filename2 ? filename2->name : "");
 	return WriteAuditLog(buf, is_granted);
 }
-#else
-static inline void AuditFileLog(const struct path_info *filename, const u8 perm, const int is_granted) {}
-static inline void AuditWriteLog(const char *operation, const struct path_info *filename1, const struct path_info *filename2, const int is_granted) {}
-#endif
-
-/*************************  PERMISSION MAP HANDLER  *************************/
-
-int SetPermissionMapping(IO_BUFFER *head)
-{
-	int i;
-	char *data = head->write_buf;
-	char *cp = NULL;
-	if ((cp = strchr(data, '=')) == NULL) {
-	out: ;
-		printk("ERROR: Invalid line '%s=%s'\n", data, cp);
-		printk("This line must be one of the following. The first is the default.\n");
-		printk("%s=%s if you want to check this permission using this permission.\n", data, data);
-		printk("%s=generic-write if you want to check this permission using generic-write permission.\n", data);
-		printk("%s=no-check if you don't want to check this permission.\n", data);
-		return -EINVAL;
-	}
-	*cp++ = '\0';
-	for (i = 0; acl_type_array[i].keyword; i++) {
-		if (strcmp(acl_type_array[i].keyword, data)) continue;
-		if (strcmp(cp, acl_type_array[i].keyword) == 0) acl_type_array[i].check_type = 1;
-		else if (strcmp(cp, "generic-write") == 0) acl_type_array[i].check_type = 0;
-		else if (strcmp(cp, "no-check") == 0) acl_type_array[i].check_type = -1;
-		else goto out;
-		return 0;
-	}
-	printk("WARNING: Unprocessed line '%s=%s'\n", data, cp);
-	return -EINVAL;
-}
-
-int ReadPermissionMapping(IO_BUFFER *head)
-{
-	if (!head->read_eof) {
-		int i;
-		for (i = 0; acl_type_array[i].keyword; i++) {
-			io_printf(head, "%s=%s\n", acl_type_array[i].keyword, acl_type_array[i].check_type > 0 ? acl_type_array[i].keyword : acl_type_array[i].check_type == 0 ? "generic-write" : "no-check");
-		}
-		head->read_eof = 1;
-	}
-	return 0;
-}
 
 /*************************  GLOBALLY READABLE FILE HANDLER  *************************/
 
-static GLOBALLY_READABLE_FILE_ENTRY *globally_readable_list = NULL;
+static struct globally_readable_file_entry *globally_readable_list = NULL;
 
 static int AddGloballyReadableEntry(const char *filename, const int is_delete)
 {
-	GLOBALLY_READABLE_FILE_ENTRY *new_entry, *ptr;
+	struct globally_readable_file_entry *new_entry, *ptr;
 	static DECLARE_MUTEX(lock);
 	const struct path_info *saved_filename;
 	int error = -ENOMEM;
@@ -218,9 +162,9 @@ static int AddGloballyReadableEntry(const char *filename, const int is_delete)
 	if (is_delete) {
 		error = -ENOENT; goto out;
 	}
-	if ((new_entry = (GLOBALLY_READABLE_FILE_ENTRY *) alloc_element(sizeof(GLOBALLY_READABLE_FILE_ENTRY))) == NULL) goto out;
+	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->filename = saved_filename;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = globally_readable_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -234,7 +178,7 @@ static int AddGloballyReadableEntry(const char *filename, const int is_delete)
 
 static int IsGloballyReadableFile(const struct path_info *filename)
 {
-	GLOBALLY_READABLE_FILE_ENTRY *ptr;
+	struct globally_readable_file_entry *ptr;
 	for (ptr = globally_readable_list; ptr; ptr = ptr->next) {
 		if (!ptr->is_deleted && !pathcmp(filename, ptr->filename)) return 1;
 	}
@@ -246,12 +190,12 @@ int AddGloballyReadablePolicy(char *filename, const int is_delete)
 	return AddGloballyReadableEntry(filename, is_delete);
 }
 
-int ReadGloballyReadablePolicy(IO_BUFFER *head)
+int ReadGloballyReadablePolicy(struct io_buffer *head)
 {
-	GLOBALLY_READABLE_FILE_ENTRY *ptr = (GLOBALLY_READABLE_FILE_ENTRY *) head->read_var2;
+	struct globally_readable_file_entry *ptr = head->read_var2;
 	if (!ptr) ptr = globally_readable_list;
 	while (ptr) {
-		head->read_var2 = (void *) ptr;
+		head->read_var2 = ptr;
 		if (!ptr->is_deleted && io_printf(head, KEYWORD_ALLOW_READ "%s\n", ptr->filename->name)) break;
 		ptr = ptr->next;
 	}
@@ -260,17 +204,21 @@ int ReadGloballyReadablePolicy(IO_BUFFER *head)
 
 /*************************  FILE GROUP HANDLER  *************************/
 
-static GROUP_ENTRY *group_list = NULL;
+static struct group_entry *group_list = NULL;
 
 static int AddGroupEntry(const char *group_name, const char *member_name, const int is_delete)
 {
 	static DECLARE_MUTEX(lock);
-	GROUP_ENTRY *new_group, *group;
-	GROUP_MEMBER *new_member, *member;
+	struct group_entry *new_group, *group;
+	struct group_member *new_member, *member;
 	const struct path_info *saved_group_name, *saved_member_name;
 	int error = -ENOMEM;
 	if (!IsCorrectPath(group_name, 0, 0, 0, __FUNCTION__) || !group_name[0] ||
 		!IsCorrectPath(member_name, 0, 0, 0, __FUNCTION__) || !member_name[0]) return -EINVAL;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 22)
+	if (!strcmp(member_name, "pipe:"))
+		member_name = "pipe:[\\$]";
+#endif
 	if ((saved_group_name = SaveName(group_name)) == NULL ||
 		(saved_member_name = SaveName(member_name)) == NULL) return -ENOMEM;
 	down(&lock);
@@ -290,9 +238,9 @@ static int AddGroupEntry(const char *group_name, const char *member_name, const
 		goto out;
 	}
 	if (!group) {
-		if ((new_group = (GROUP_ENTRY *) alloc_element(sizeof(GROUP_ENTRY))) == NULL) goto out;
+		if ((new_group = alloc_element(sizeof(*new_group))) == NULL) goto out;
 		new_group->group_name = saved_group_name;
-		mb(); /* Instead of using spinlock. */
+		mb(); /* Avoid out-of-order execution. */
 		if ((group = group_list) != NULL) {
 			while (group->next) group = group->next; group->next = new_group;
 		} else {
@@ -300,9 +248,9 @@ static int AddGroupEntry(const char *group_name, const char *member_name, const
 		}
 		group = new_group;
 	}
-	if ((new_member = (GROUP_MEMBER *) alloc_element(sizeof(GROUP_MEMBER))) == NULL) goto out;
+	if ((new_member = alloc_element(sizeof(*new_member))) == NULL) goto out;
 	new_member->member_name = saved_member_name;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((member = group->first_member) != NULL) {
 		while (member->next) member = member->next; member->next = new_member;
 	} else {
@@ -322,10 +270,10 @@ int AddGroupPolicy(char *data, const int is_delete)
 	return AddGroupEntry(data, cp, is_delete);
 }
 
-static GROUP_ENTRY *FindOrAssignNewGroup(const char *group_name)
+static struct group_entry *FindOrAssignNewGroup(const char *group_name)
 {
 	int i;
-	GROUP_ENTRY *group;
+	struct group_entry *group;
 	for (i = 0; i <= 1; i++) {
 		for (group = group_list; group; group = group->next) {
 			if (strcmp(group_name, group->group_name->name) == 0) return group;
@@ -338,9 +286,9 @@ static GROUP_ENTRY *FindOrAssignNewGroup(const char *group_name)
 	return NULL;
 }
 
-static int PathMatchesToGroup(const struct path_info *pathname, const GROUP_ENTRY *group, const int may_use_pattern)
+static int PathMatchesToGroup(const struct path_info *pathname, const struct group_entry *group, const int may_use_pattern)
 {
-	GROUP_MEMBER *member;
+	struct group_member *member;
 	for (member = group->first_member; member; member = member->next) {
 		if (member->is_deleted) continue;
 		if (!member->member_name->is_patterned) {
@@ -352,16 +300,16 @@ static int PathMatchesToGroup(const struct path_info *pathname, const GROUP_ENTR
 	return 0;
 }
 
-int ReadGroupPolicy(IO_BUFFER *head)
+int ReadGroupPolicy(struct io_buffer *head)
 {
-	GROUP_ENTRY *group = (GROUP_ENTRY *) head->read_var1;
-	GROUP_MEMBER *member = (GROUP_MEMBER *) head->read_var2;
+	struct group_entry *group = head->read_var1;
+	struct group_member *member = head->read_var2;
 	if (!group) group = group_list;
 	while (group) {
-		head->read_var1 = (struct domain_info *) group;
+		head->read_var1 = group;
 		if (!member) member = group->first_member;
 		while (member) {
-			head->read_var2 = (void *) member;
+			head->read_var2 = member;
 			if (!member->is_deleted && io_printf(head, KEYWORD_PATH_GROUP "%s %s\n", group->group_name->name, member->member_name->name)) break;
 			member = member->next;
 		}
@@ -374,11 +322,11 @@ int ReadGroupPolicy(IO_BUFFER *head)
 
 /*************************  FILE PATTERN HANDLER  *************************/
 
-static PATTERN_ENTRY *pattern_list = NULL;
+static struct pattern_entry *pattern_list = NULL;
 
 static int AddPatternEntry(const char *pattern, const int is_delete)
 {
-	PATTERN_ENTRY *new_entry, *ptr;
+	struct pattern_entry *new_entry, *ptr;
 	static DECLARE_MUTEX(lock);
 	const struct path_info *saved_pattern;
 	int error = -ENOMEM;
@@ -396,9 +344,9 @@ static int AddPatternEntry(const char *pattern, const int is_delete)
 		error = -ENOENT;
 		goto out;
 	}
-	if ((new_entry = (PATTERN_ENTRY *) alloc_element(sizeof(PATTERN_ENTRY))) == NULL) goto out;
+	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->pattern = saved_pattern;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = pattern_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -412,7 +360,7 @@ static int AddPatternEntry(const char *pattern, const int is_delete)
 
 static const struct path_info *GetPattern(const struct path_info *filename)
 {
-	PATTERN_ENTRY *ptr;
+	struct pattern_entry *ptr;
 	const struct path_info *pattern = NULL;
 	for (ptr = pattern_list; ptr; ptr = ptr->next) {
 		if (ptr->is_deleted) continue;
@@ -434,12 +382,12 @@ int AddPatternPolicy(char *pattern, const int is_delete)
 	return AddPatternEntry(pattern, is_delete);
 }
 
-int ReadPatternPolicy(IO_BUFFER *head)
+int ReadPatternPolicy(struct io_buffer *head)
 {
-	PATTERN_ENTRY *ptr = (PATTERN_ENTRY *) head->read_var2;
+	struct pattern_entry *ptr = head->read_var2;
 	if (!ptr) ptr = pattern_list;
 	while (ptr) {
-		head->read_var2 = (void *) ptr;
+		head->read_var2 = ptr;
 		if (!ptr->is_deleted && io_printf(head, KEYWORD_FILE_PATTERN "%s\n", ptr->pattern->name)) break;
 		ptr = ptr->next;
 	}
@@ -448,11 +396,11 @@ int ReadPatternPolicy(IO_BUFFER *head)
 
 /*************************  NON REWRITABLE FILE HANDLER  *************************/
 
-static NO_REWRITE_ENTRY *no_rewrite_list = NULL;
+static struct no_rewrite_entry *no_rewrite_list = NULL;
 
 static int AddNoRewriteEntry(const char *pattern, const int is_delete)
 {
-	NO_REWRITE_ENTRY *new_entry, *ptr;
+	struct no_rewrite_entry *new_entry, *ptr;
 	static DECLARE_MUTEX(lock);
 	const struct path_info *saved_pattern;
 	int error = -ENOMEM;
@@ -470,9 +418,9 @@ static int AddNoRewriteEntry(const char *pattern, const int is_delete)
 		error = -ENOENT;
 		goto out;
 	}
-	if ((new_entry = (NO_REWRITE_ENTRY *) alloc_element(sizeof(NO_REWRITE_ENTRY))) == NULL) goto out;
+	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->pattern = saved_pattern;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = no_rewrite_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -486,7 +434,7 @@ static int AddNoRewriteEntry(const char *pattern, const int is_delete)
 
 static int IsNoRewriteFile(const struct path_info *filename)
 {
-	NO_REWRITE_ENTRY *ptr;
+	struct no_rewrite_entry *ptr;
 	for (ptr = no_rewrite_list; ptr; ptr = ptr->next) {
 		if (ptr->is_deleted) continue;
 		if (!PathMatchesToPattern(filename, ptr->pattern)) continue;
@@ -500,12 +448,12 @@ int AddNoRewritePolicy(char *pattern, const int is_delete)
 	return AddNoRewriteEntry(pattern, is_delete);
 }
 
-int ReadNoRewritePolicy(IO_BUFFER *head)
+int ReadNoRewritePolicy(struct io_buffer *head)
 {
-	NO_REWRITE_ENTRY *ptr = (NO_REWRITE_ENTRY *) head->read_var2;
+	struct no_rewrite_entry *ptr = head->read_var2;
 	if (!ptr) ptr = no_rewrite_list;
 	while (ptr) {
-		head->read_var2 = (void *) ptr;
+		head->read_var2 = ptr;
 		if (!ptr->is_deleted && io_printf(head, KEYWORD_DENY_REWRITE "%s\n", ptr->pattern->name)) break;
 		ptr = ptr->next;
 	}
@@ -514,7 +462,7 @@ int ReadNoRewritePolicy(IO_BUFFER *head)
 
 /*************************  FILE ACL HANDLER  *************************/
 
-static int AddFileACL(const char *filename, u8 perm, struct domain_info * const domain, const u8 is_add, const struct condition_list *condition)
+static int AddFileACL(const char *filename, u8 perm, struct domain_info * const domain, const struct condition_list *condition, const u8 is_delete)
 {
 	const struct path_info *saved_filename;
 	struct acl_info *ptr;
@@ -531,29 +479,32 @@ static int AddFileACL(const char *filename, u8 perm, struct domain_info * const
 		if ((saved_filename = (struct path_info *) FindOrAssignNewGroup(filename + 1)) == NULL) return -ENOMEM;
 		is_group = 1;
 	} else {
+		if (strendswith(filename, "/")) {
+			return 0; /* Valid permissions for directory are only 'allow_mkdir' and 'allow_rmdir'. */
+		}
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 22)
+		if (!strcmp(filename, "pipe:"))
+			filename = "pipe:[\\$]";
+#endif
 		if ((saved_filename = SaveName(filename)) == NULL) return -ENOMEM;
-		if (!saved_filename->is_dir) {
-			if (perm == 4 && IsGloballyReadableFile(saved_filename) && is_add) {
-				return 0;   /* Don't add if the file is globally readable files. */
-			}
-		} else {
-			if ((perm & 2) == 0) return 0; /* Don't add if the directory doesn't have write permission. */
+		if (!is_delete && perm == 4 && IsGloballyReadableFile(saved_filename)) {
+			return 0;   /* Don't add if the file is globally readable files. */
 		}
 	}
 	down(&domain_acl_lock);
-	if (is_add) {
+	if (!is_delete) {
 		if ((ptr = domain->first_acl_ptr) == NULL) goto first_entry;
 		while (1) {
-			FILE_ACL_RECORD *new_ptr;
+			struct file_acl_record *new_ptr = (struct file_acl_record *) ptr;
 			if (ptr->type == TYPE_FILE_ACL && ptr->cond == condition) {
-				if (((FILE_ACL_RECORD *) ptr)->u.filename == saved_filename) {
+				if (new_ptr->u.filename == saved_filename) {
 					if (ptr->is_deleted) {
-						ptr->u.b[0] = 0;
+						new_ptr->perm = 0;
 						mb();
 						ptr->is_deleted = 0;
 					}
 					/* Found. Just 'OR' the permission bits. */
-					ptr->u.b[0] |= perm;
+					new_ptr->perm |= perm;
 					error = 0;
 					UpdateCounter(CCS_UPDATES_COUNTER_DOMAIN_POLICY);
 					break;
@@ -564,12 +515,11 @@ static int AddFileACL(const char *filename, u8 perm, struct domain_info * const
 				continue;
 			}
 	first_entry: ;
-			if (is_add == 1 && TooManyDomainACL(domain)) break;
 			/* Not found. Append it to the tail. */
-			if ((new_ptr = (FILE_ACL_RECORD *) alloc_element(sizeof(FILE_ACL_RECORD))) == NULL) break;
+			if ((new_ptr = alloc_element(sizeof(*new_ptr))) == NULL) break;
 			new_ptr->head.type = TYPE_FILE_ACL;
-			new_ptr->head.u.b[0] = perm;
-			new_ptr->head.u.b[1] = is_group;
+			new_ptr->perm = perm;
+			new_ptr->u_is_group = is_group;
 			new_ptr->head.cond = condition;
 			new_ptr->u.filename = saved_filename;
 			error = AddDomainACL(ptr, domain, (struct acl_info *) new_ptr);
@@ -577,8 +527,9 @@ static int AddFileACL(const char *filename, u8 perm, struct domain_info * const
 		}
 	} else {
 		for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
-			if (ptr->type != TYPE_FILE_ACL || ptr->is_deleted || ptr->cond != condition || ptr->u.b[0] != perm) continue;
-			if (((FILE_ACL_RECORD *) ptr)->u.filename != saved_filename) continue;
+			struct file_acl_record *ptr2 = (struct file_acl_record *) ptr;
+			if (ptr->type != TYPE_FILE_ACL || ptr->is_deleted || ptr->cond != condition || ptr2->perm != perm) continue;
+			if (ptr2->u.filename != saved_filename) continue;
 			error = DelDomainACL(ptr);
 			break;
 		}
@@ -597,11 +548,12 @@ static int CheckFileACL(const struct path_info *filename, const u8 perm, struct
 		if (perm == 4 && IsGloballyReadableFile(filename)) return 0;
 	}
 	for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
-		if (ptr->type != TYPE_FILE_ACL || ptr->is_deleted || (ptr->u.b[0] & perm) != perm || CheckCondition(ptr->cond, obj)) continue;
-		if (ptr->u.b[1]) {
-			if (PathMatchesToGroup(filename, ((FILE_ACL_RECORD *) ptr)->u.group, may_use_pattern)) return 0;
-		} else if (may_use_pattern || !((FILE_ACL_RECORD *) ptr)->u.filename->is_patterned) {
-			if (PathMatchesToPattern(filename, ((FILE_ACL_RECORD *) ptr)->u.filename)) return 0;
+		struct file_acl_record *ptr2 = (struct file_acl_record *) ptr;
+		if (ptr->type != TYPE_FILE_ACL || ptr->is_deleted || (ptr2->perm & perm) != perm || CheckCondition(ptr->cond, obj)) continue;
+		if (ptr2->u_is_group) {
+			if (PathMatchesToGroup(filename, ptr2->u.group, may_use_pattern)) return 0;
+		} else if (may_use_pattern || !ptr2->u.filename->is_patterned) {
+			if (PathMatchesToPattern(filename, ptr2->u.filename)) return 0;
 		}
 	}
 	return -EPERM;
@@ -620,10 +572,10 @@ static int CheckFilePerm2(const struct path_info *filename, const u8 perm, const
 			printk("TOMOYO-%s: Access %d(%s) to %s denied for %s\n", GetMSG(is_enforce), perm, operation, filename->name, GetLastName(domain));
 		}
 		if (is_enforce) error = CheckSupervisor("%s\n%d %s\n", domain->domainname->name, perm, filename->name);
-		else if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_FILE)) {
+		else if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_FILE, domain)) {
 			/* Don't use patterns if execution bit is on. */
 			const struct path_info *patterned_file = ((perm & 1) == 0) ? GetPattern(filename) : filename;
-			AddFileACL(patterned_file->name, perm, domain, 1, NULL);
+			AddFileACL(patterned_file->name, perm, domain, NULL, 0);
 		}
 		if (!is_enforce) error = 0;
 	}
@@ -642,7 +594,7 @@ int AddFilePolicy(char *data, struct domain_info *domain, const int is_delete)
 	if (sscanf(data, "%u", &perm) == 1) {
 		cp = FindConditionPart(filename);
 		if (cp && (condition = FindOrAssignNewCondition(cp)) == NULL) goto out;
-		return AddFileACL(filename, (u8) perm, domain, is_delete ? 0 : -1, condition);
+		return AddFileACL(filename, (u8) perm, domain, condition, is_delete);
 	}
 	if (strncmp(data, "allow_", 6)) goto out;
 	data += 6;
@@ -654,11 +606,11 @@ int AddFilePolicy(char *data, struct domain_info *domain, const int is_delete)
 			*filename2++ = '\0';
 			cp = FindConditionPart(filename2);
 			if (cp && (condition = FindOrAssignNewCondition(cp)) == NULL) goto out;
-			return AddDoubleWriteACL(type, filename, filename2, domain, is_delete ? 0 : -1, condition);
+			return AddDoubleWriteACL(type, filename, filename2, domain, condition, is_delete);
 		} else {
 			cp = FindConditionPart(filename);
 			if (cp && (condition = FindOrAssignNewCondition(cp)) == NULL) goto out;
-			return AddSingleWriteACL(type, filename, domain, is_delete ? 0 : -1, condition);
+			return AddSingleWriteACL(type, filename, domain, condition, is_delete);
 		}
 		break;
 	}
@@ -666,7 +618,7 @@ int AddFilePolicy(char *data, struct domain_info *domain, const int is_delete)
 	return -EINVAL;
 }
 
-static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_info * const domain, const u8 is_add, const struct condition_list *condition)
+static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_info * const domain, const struct condition_list *condition, const u8 is_delete)
 {
 	const struct path_info *saved_filename;
 	struct acl_info *ptr;
@@ -682,12 +634,12 @@ static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_
 		if ((saved_filename = SaveName(filename)) == NULL) return -ENOMEM;
 	}
 	down(&domain_acl_lock);
-	if (is_add) {
+	if (!is_delete) {
 		if ((ptr = domain->first_acl_ptr) == NULL) goto first_entry;
 		while (1) {
-			SINGLE_ACL_RECORD *new_ptr;
+			struct single_acl_record *new_ptr = (struct single_acl_record *) ptr;
 			if (ptr->type == type && ptr->cond == condition) {
-				if (((SINGLE_ACL_RECORD *) ptr)->u.filename == saved_filename) {
+				if (new_ptr->u.filename == saved_filename) {
 					ptr->is_deleted = 0;
 					/* Found. Nothing to do. */
 					error = 0;
@@ -699,12 +651,11 @@ static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_
 				continue;
 			}
 		first_entry: ;
-			if (is_add == 1 && TooManyDomainACL(domain)) break;
 			/* Not found. Append it to the tail. */
-			if ((new_ptr = (SINGLE_ACL_RECORD *) alloc_element(sizeof(SINGLE_ACL_RECORD))) == NULL) break;
+			if ((new_ptr = alloc_element(sizeof(*new_ptr))) == NULL) break;
 			new_ptr->head.type = type;
 			new_ptr->head.cond = condition;
-			new_ptr->head.u.w = is_group;
+			new_ptr->u_is_group = is_group;
 			new_ptr->u.filename = saved_filename;
 			error = AddDomainACL(ptr, domain, (struct acl_info *) new_ptr);
 			break;
@@ -712,8 +663,9 @@ static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_
 	} else {
 		error = -ENOENT;
 		for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
+			struct single_acl_record *ptr2 = (struct single_acl_record *) ptr;
 			if (ptr->type != type || ptr->is_deleted || ptr->cond != condition) continue;
-			if (((SINGLE_ACL_RECORD *) ptr)->u.filename != saved_filename) continue;
+			if (ptr2->u.filename != saved_filename) continue;
 			error = DelDomainACL(ptr);
 			break;
 		}
@@ -722,7 +674,7 @@ static int AddSingleWriteACL(const u8 type, const char *filename, struct domain_
 	return error;
 }
 
-static int AddDoubleWriteACL(const u8 type, const char *filename1, const char *filename2, struct domain_info * const domain, const u8 is_add, const struct condition_list *condition)
+static int AddDoubleWriteACL(const u8 type, const char *filename1, const char *filename2, struct domain_info * const domain, const struct condition_list *condition, const u8 is_delete)
 {
 	const struct path_info *saved_filename1, *saved_filename2;
 	struct acl_info *ptr;
@@ -745,12 +697,12 @@ static int AddDoubleWriteACL(const u8 type, const char *filename1, const char *f
 		if ((saved_filename2 = SaveName(filename2)) == NULL) return -ENOMEM;
 	}
 	down(&domain_acl_lock);
-	if (is_add) {
+	if (!is_delete) {
 		if ((ptr = domain->first_acl_ptr) == NULL) goto first_entry;
 		while (1) {
-			DOUBLE_ACL_RECORD *new_ptr;
+			struct double_acl_record *new_ptr = (struct double_acl_record *) ptr;
 			if (ptr->type == type && ptr->cond == condition) {
-				if (((DOUBLE_ACL_RECORD *) ptr)->u1.filename1 == saved_filename1 && ((DOUBLE_ACL_RECORD *) ptr)->u2.filename2 == saved_filename2) {
+				if (new_ptr->u1.filename1 == saved_filename1 && new_ptr->u2.filename2 == saved_filename2) {
 					ptr->is_deleted = 0;
 					/* Found. Nothing to do. */
 					error = 0;
@@ -762,13 +714,12 @@ static int AddDoubleWriteACL(const u8 type, const char *filename1, const char *f
 				continue;
 			}
 		first_entry: ;
-			if (is_add == 1 && TooManyDomainACL(domain)) break;
 			/* Not found. Append it to the tail. */
-			if ((new_ptr = (DOUBLE_ACL_RECORD *) alloc_element(sizeof(DOUBLE_ACL_RECORD))) == NULL) break;
+			if ((new_ptr = alloc_element(sizeof(*new_ptr))) == NULL) break;
 			new_ptr->head.type = type;
 			new_ptr->head.cond = condition;
-			new_ptr->head.u.b[0] = is_group1;
-			new_ptr->head.u.b[1] = is_group2;
+			new_ptr->u1_is_group = is_group1;
+			new_ptr->u2_is_group = is_group2;
 			new_ptr->u1.filename1 = saved_filename1;
 			new_ptr->u2.filename2 = saved_filename2;
 			error = AddDomainACL(ptr, domain, (struct acl_info *) new_ptr);
@@ -777,9 +728,9 @@ static int AddDoubleWriteACL(const u8 type, const char *filename1, const char *f
 	} else {
 		error = -ENOENT;
 		for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
+			struct double_acl_record *ptr2 = (struct double_acl_record *) ptr;
 			if (ptr->type != type || ptr->is_deleted || ptr->cond != condition) continue;
-			if (((DOUBLE_ACL_RECORD *) ptr)->u1.filename1 != saved_filename1 ||
-				((DOUBLE_ACL_RECORD *) ptr)->u2.filename2 != saved_filename2) continue;
+			if (ptr2->u1.filename1 != saved_filename1 || ptr2->u2.filename2 != saved_filename2) continue;
 			error = DelDomainACL(ptr);
 			break;
 		}
@@ -794,11 +745,12 @@ static int CheckSingleWriteACL(const u8 type, const struct path_info *filename,
 	struct acl_info *ptr;
 	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
 	for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
+		struct single_acl_record *ptr2 = (struct single_acl_record *) ptr;
 		if (ptr->type != type || ptr->is_deleted || CheckCondition(ptr->cond, obj)) continue;
-		if (ptr->u.w) {
-			if (!PathMatchesToGroup(filename, ((SINGLE_ACL_RECORD *) ptr)->u.group, 1)) continue;
+		if (ptr2->u_is_group) {
+			if (!PathMatchesToGroup(filename, ptr2->u.group, 1)) continue;
 		} else {
-			if (!PathMatchesToPattern(filename, ((SINGLE_ACL_RECORD *) ptr)->u.filename)) continue;
+			if (!PathMatchesToPattern(filename, ptr2->u.filename)) continue;
 		}
 		return 0;
 	}
@@ -811,16 +763,17 @@ static int CheckDoubleWriteACL(const u8 type, const struct path_info *filename1,
 	struct acl_info *ptr;
 	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
 	for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
+		struct double_acl_record *ptr2 = (struct double_acl_record *) ptr;
 		if (ptr->type != type || ptr->is_deleted || CheckCondition(ptr->cond, obj)) continue;
-		if (ptr->u.b[0]) {
-			if (!PathMatchesToGroup(filename1, ((DOUBLE_ACL_RECORD *) ptr)->u1.group1, 1)) continue;
+		if (ptr2->u1_is_group) {
+			if (!PathMatchesToGroup(filename1, ptr2->u1.group1, 1)) continue;
 		} else {
-			if (!PathMatchesToPattern(filename1, ((DOUBLE_ACL_RECORD *) ptr)->u1.filename1)) continue;
+			if (!PathMatchesToPattern(filename1, ptr2->u1.filename1)) continue;
 		}
-		if (ptr->u.b[1]) {
-			if (!PathMatchesToGroup(filename2, ((DOUBLE_ACL_RECORD *) ptr)->u2.group2, 1)) continue;
+		if (ptr2->u2_is_group) {
+			if (!PathMatchesToGroup(filename2, ptr2->u2.group2, 1)) continue;
 		} else {
-			if (!PathMatchesToPattern(filename2, ((DOUBLE_ACL_RECORD *) ptr)->u2.filename2)) continue;
+			if (!PathMatchesToPattern(filename2, ptr2->u2.filename2)) continue;
 		}
 		return 0;
 	}
@@ -833,20 +786,15 @@ static int CheckSingleWritePermission2(const unsigned int operation, const struc
 	struct domain_info * const domain = current->domain_info;
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_FILE);
 	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
-	if (CheckACLFlags(operation) < 0) return 0;
-	if (CheckACLFlags(operation) > 0) {
-		error = CheckSingleWriteACL(operation, filename, obj);
-		AuditWriteLog(acltype2keyword(operation), filename, NULL, !error);
-		if (error) {
-			if (TomoyoVerboseMode()) {
-				printk("TOMOYO-%s: Access '%s %s' denied for %s\n", GetMSG(is_enforce), acltype2keyword(operation), filename->name, GetLastName(domain));
-			}
-			if (is_enforce) error = CheckSupervisor("%s\nallow_%s %s\n", domain->domainname->name, acltype2keyword(operation), filename->name);
-			else if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_FILE)) AddSingleWriteACL(operation, GetPattern(filename)->name, domain, 1, NULL);
-			if (!is_enforce) error = 0;
+	error = CheckSingleWriteACL(operation, filename, obj);
+	AuditWriteLog(acltype2keyword(operation), filename, NULL, !error);
+	if (error) {
+		if (TomoyoVerboseMode()) {
+			printk("TOMOYO-%s: Access '%s %s' denied for %s\n", GetMSG(is_enforce), acltype2keyword(operation), filename->name, GetLastName(domain));
 		}
-	} else {
-		error = CheckFilePerm2(filename, 2, acltype2keyword(operation), obj);
+		if (is_enforce) error = CheckSupervisor("%s\nallow_%s %s\n", domain->domainname->name, acltype2keyword(operation), filename->name);
+		else if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_FILE, domain)) AddSingleWriteACL(operation, GetPattern(filename)->name, domain, NULL, 0);
+		if (!is_enforce) error = 0;
 	}
 	if (!error && operation == TYPE_TRUNCATE_ACL && IsNoRewriteFile(filename)) {
 		error = CheckSingleWritePermission2(TYPE_REWRITE_ACL, filename, obj);
@@ -879,7 +827,7 @@ int CheckOpenPermission(struct dentry *dentry, struct vfsmount *mnt, const int f
 	int error = -ENOMEM;
 	struct path_info *buf;
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_FILE);
-	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
+	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE) || !mnt) return 0;
 	if (acc_mode == 0) return 0;
 	if (dentry->d_inode && S_ISDIR(dentry->d_inode->i_mode)) {
 		/* I don't check directories here because mkdir() and rmdir() don't call me. */
@@ -912,7 +860,7 @@ int CheckSingleWritePermission(const unsigned int operation, struct dentry *dent
 	int error = -ENOMEM;
 	struct path_info *buf;
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_FILE);
-	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
+	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE) || !mnt) return 0;
 	buf = GetPath(dentry, mnt);
 	if (buf) {
 		struct obj_info obj;
@@ -933,12 +881,15 @@ int CheckSingleWritePermission(const unsigned int operation, struct dentry *dent
 	if (!is_enforce) error = 0;
 	return error;
 }
+EXPORT_SYMBOL(CheckSingleWritePermission);
 
 int CheckReWritePermission(struct file *filp)
 {
 	int error = -ENOMEM;
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_FILE);
-	struct path_info *buf = GetPath(filp->f_dentry, filp->f_vfsmnt);
+	struct path_info *buf;
+	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE) || !filp->f_vfsmnt) return 0;
+	buf = GetPath(filp->f_dentry, filp->f_vfsmnt);
 	if (buf) {
 		if (IsNoRewriteFile(buf)) {
 			struct obj_info obj;
@@ -961,8 +912,7 @@ int CheckDoubleWritePermission(const unsigned int operation, struct dentry *dent
 	struct path_info *buf1, *buf2;
 	struct domain_info * const domain = current->domain_info;
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_FILE);
-	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE)) return 0;
-	if (CheckACLFlags(operation) < 0) return 0;		
+	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_FILE) || !mnt1 || !mnt2) return 0;
 	buf1 = GetPath(dentry1, mnt1);
 	buf2 = GetPath(dentry2, mnt2);
 	if (buf1 && buf2) {
@@ -984,19 +934,14 @@ int CheckDoubleWritePermission(const unsigned int operation, struct dentry *dent
 		obj.path1_vfsmnt = mnt1;
 		obj.path2_dentry = dentry2;
 		obj.path2_vfsmnt = mnt2;
-		if (CheckACLFlags(operation) > 0) {
-			error = CheckDoubleWriteACL(operation, buf1, buf2, &obj);
-			AuditWriteLog(acltype2keyword(operation), buf1, buf2, !error);
-			if (error) {
-				if (TomoyoVerboseMode()) {
-					printk("TOMOYO-%s: Access '%s %s %s' denied for %s\n", GetMSG(is_enforce), acltype2keyword(operation), buf1->name, buf2->name, GetLastName(domain));
-				}
-				if (is_enforce) error = CheckSupervisor("%s\nallow_%s %s %s\n", domain->domainname->name, acltype2keyword(operation), buf1->name, buf2->name);
-				else if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_FILE)) AddDoubleWriteACL(operation, GetPattern(buf1)->name, GetPattern(buf2)->name, domain, 1, NULL);
+		error = CheckDoubleWriteACL(operation, buf1, buf2, &obj);
+		AuditWriteLog(acltype2keyword(operation), buf1, buf2, !error);
+		if (error) {
+			if (TomoyoVerboseMode()) {
+				printk("TOMOYO-%s: Access '%s %s %s' denied for %s\n", GetMSG(is_enforce), acltype2keyword(operation), buf1->name, buf2->name, GetLastName(domain));
 			}
-		} else {
-			error = CheckFilePerm2(buf1, 2, acltype2keyword(operation), &obj);
-			if (!error) error = CheckFilePerm2(buf2, 2, acltype2keyword(operation), &obj);
+			if (is_enforce) error = CheckSupervisor("%s\nallow_%s %s %s\n", domain->domainname->name, acltype2keyword(operation), buf1->name, buf2->name);
+			else if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_FILE, domain)) AddDoubleWriteACL(operation, GetPattern(buf1)->name, GetPattern(buf2)->name, domain, NULL, 0);
 		}
 	}
 	ccs_free(buf1);
@@ -1005,11 +950,4 @@ int CheckDoubleWritePermission(const unsigned int operation, struct dentry *dent
 	return error;
 }
 
-EXPORT_SYMBOL(CheckFilePerm);
-EXPORT_SYMBOL(CheckExecPerm);
-EXPORT_SYMBOL(CheckOpenPermission);
-EXPORT_SYMBOL(CheckReWritePermission);
-EXPORT_SYMBOL(CheckSingleWritePermission);
-EXPORT_SYMBOL(CheckDoubleWritePermission);
-
 /***** TOMOYO Linux end. *****/
